@@ -1,6 +1,6 @@
 /*
   KeePass Password Safe - The Open-Source Password Manager
-  Copyright (C) 2003-2021 Dominik Reichl <dominik.reichl@t-online.de>
+  Copyright (C) 2003-2024 Dominik Reichl <dominik.reichl@t-online.de>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -190,6 +190,17 @@ namespace KeePassLib.Utility
 			return l.ToArray();
 		}
 
+		internal static byte[] ParseBase32(string str, bool bAutoPad)
+		{
+			if(str == null) { Debug.Assert(false); return null; }
+
+			// https://sourceforge.net/p/keepass/discussion/329220/thread/59b61fddea/
+			if(bAutoPad && ((str.Length % 8) != 0))
+				str = str.PadRight((str.Length & ~7) + 8, '=');
+
+			return ParseBase32(str);
+		}
+
 		/// <summary>
 		/// Set all bytes in a byte array to zero.
 		/// </summary>
@@ -215,6 +226,33 @@ namespace KeePassLib.Utility
 			Array.Clear(v, 0, v.Length);
 		}
 
+		private static byte[] g_pbZero = null;
+		[MethodImpl(MioNoOptimize)]
+		public static void ZeroMemory(IntPtr pb, long cb)
+		{
+			if(pb == IntPtr.Zero) { Debug.Assert(false); return; }
+			if(cb < 0) { Debug.Assert(false); return; }
+
+			byte[] pbZero = g_pbZero;
+			if(pbZero == null)
+			{
+				pbZero = new byte[4096];
+				g_pbZero = pbZero;
+			}
+
+			long cbZero = pbZero.Length;
+
+			while(cb != 0)
+			{
+				long cbBlock = Math.Min(cb, cbZero);
+
+				Marshal.Copy(pbZero, 0, pb, (int)cbBlock);
+
+				pb = AddPtr(pb, cbBlock);
+				cb -= cbBlock;
+			}
+		}
+
 		/// <summary>
 		/// Convert 2 bytes to a 16-bit unsigned integer (little-endian).
 		/// </summary>
@@ -331,15 +369,7 @@ namespace KeePassLib.Utility
 		/// </summary>
 		public static byte[] UInt16ToBytes(ushort uValue)
 		{
-			byte[] pb = new byte[2];
-
-			unchecked
-			{
-				pb[0] = (byte)uValue;
-				pb[1] = (byte)(uValue >> 8);
-			}
-
-			return pb;
+			return new byte[2] { (byte)uValue, (byte)(uValue >> 8) };
 		}
 
 		/// <summary>
@@ -347,17 +377,8 @@ namespace KeePassLib.Utility
 		/// </summary>
 		public static byte[] UInt32ToBytes(uint uValue)
 		{
-			byte[] pb = new byte[4];
-
-			unchecked
-			{
-				pb[0] = (byte)uValue;
-				pb[1] = (byte)(uValue >> 8);
-				pb[2] = (byte)(uValue >> 16);
-				pb[3] = (byte)(uValue >> 24);
-			}
-
-			return pb;
+			return new byte[4] { (byte)uValue, (byte)(uValue >> 8),
+				(byte)(uValue >> 16), (byte)(uValue >> 24) };
 		}
 
 		/// <summary>
@@ -366,19 +387,16 @@ namespace KeePassLib.Utility
 		public static void UInt32ToBytesEx(uint uValue, byte[] pb, int iOffset)
 		{
 			if(pb == null) { Debug.Assert(false); throw new ArgumentNullException("pb"); }
-			if((iOffset < 0) || ((iOffset + 3) >= pb.Length))
+			if((iOffset < 0) || (iOffset >= (pb.Length - 3)))
 			{
 				Debug.Assert(false);
 				throw new ArgumentOutOfRangeException("iOffset");
 			}
 
-			unchecked
-			{
-				pb[iOffset] = (byte)uValue;
-				pb[iOffset + 1] = (byte)(uValue >> 8);
-				pb[iOffset + 2] = (byte)(uValue >> 16);
-				pb[iOffset + 3] = (byte)(uValue >> 24);
-			}
+			pb[iOffset] = (byte)uValue;
+			pb[iOffset + 1] = (byte)(uValue >> 8);
+			pb[iOffset + 2] = (byte)(uValue >> 16);
+			pb[iOffset + 3] = (byte)(uValue >> 24);
 		}
 
 		/// <summary>
@@ -386,21 +404,10 @@ namespace KeePassLib.Utility
 		/// </summary>
 		public static byte[] UInt64ToBytes(ulong uValue)
 		{
-			byte[] pb = new byte[8];
-
-			unchecked
-			{
-				pb[0] = (byte)uValue;
-				pb[1] = (byte)(uValue >> 8);
-				pb[2] = (byte)(uValue >> 16);
-				pb[3] = (byte)(uValue >> 24);
-				pb[4] = (byte)(uValue >> 32);
-				pb[5] = (byte)(uValue >> 40);
-				pb[6] = (byte)(uValue >> 48);
-				pb[7] = (byte)(uValue >> 56);
-			}
-
-			return pb;
+			return new byte[8] { (byte)uValue, (byte)(uValue >> 8),
+				(byte)(uValue >> 16), (byte)(uValue >> 24),
+				(byte)(uValue >> 32), (byte)(uValue >> 40),
+				(byte)(uValue >> 48), (byte)(uValue >> 56) };
 		}
 
 		/// <summary>
@@ -409,23 +416,20 @@ namespace KeePassLib.Utility
 		public static void UInt64ToBytesEx(ulong uValue, byte[] pb, int iOffset)
 		{
 			if(pb == null) { Debug.Assert(false); throw new ArgumentNullException("pb"); }
-			if((iOffset < 0) || ((iOffset + 7) >= pb.Length))
+			if((iOffset < 0) || (iOffset >= (pb.Length - 7)))
 			{
 				Debug.Assert(false);
 				throw new ArgumentOutOfRangeException("iOffset");
 			}
 
-			unchecked
-			{
-				pb[iOffset] = (byte)uValue;
-				pb[iOffset + 1] = (byte)(uValue >> 8);
-				pb[iOffset + 2] = (byte)(uValue >> 16);
-				pb[iOffset + 3] = (byte)(uValue >> 24);
-				pb[iOffset + 4] = (byte)(uValue >> 32);
-				pb[iOffset + 5] = (byte)(uValue >> 40);
-				pb[iOffset + 6] = (byte)(uValue >> 48);
-				pb[iOffset + 7] = (byte)(uValue >> 56);
-			}
+			pb[iOffset] = (byte)uValue;
+			pb[iOffset + 1] = (byte)(uValue >> 8);
+			pb[iOffset + 2] = (byte)(uValue >> 16);
+			pb[iOffset + 3] = (byte)(uValue >> 24);
+			pb[iOffset + 4] = (byte)(uValue >> 32);
+			pb[iOffset + 5] = (byte)(uValue >> 40);
+			pb[iOffset + 6] = (byte)(uValue >> 48);
+			pb[iOffset + 7] = (byte)(uValue >> 56);
 		}
 
 		public static byte[] Int32ToBytes(int iValue)
@@ -473,9 +477,10 @@ namespace KeePassLib.Utility
 			// Return false if one of them is null (not comparable)!
 			if((x == null) || (y == null)) { Debug.Assert(false); return false; }
 
-			if(x.Length != y.Length) return false;
+			int cb = x.Length;
+			if(cb != y.Length) return false;
 
-			for(int i = 0; i < x.Length; ++i)
+			for(int i = 0; i < cb; ++i)
 			{
 				if(x[i] != y[i]) return false;
 			}
@@ -521,25 +526,24 @@ namespace KeePassLib.Utility
 			ulong h = hI;
 
 			for(int i = iOffset; i < m4; i += 4)
-				h = (h ^ pb[i] ^ ((ulong)pb[i + 1] << 8) ^
-					((ulong)pb[i + 2] << 16) ^ ((ulong)pb[i + 3] << 24)) *
-					0x5EA4A1E35C8ACDA3UL;
+				h = (pb[i] ^ ((ulong)pb[i + 1] << 8) ^ ((ulong)pb[i + 2] << 16) ^
+					((ulong)pb[i + 3] << 24) ^ h) * 0x5EA4A1E35C8ACDA3UL;
 
 			switch(cbR)
 			{
 				case 1:
 					Debug.Assert(m4 == (m - 1));
-					h = (h ^ pb[m4]) * 0x54A1CC5970AF27BBUL;
+					h = (pb[m4] ^ h) * 0x54A1CC5970AF27BBUL;
 					break;
 				case 2:
 					Debug.Assert(m4 == (m - 2));
-					h = (h ^ pb[m4] ^ ((ulong)pb[m4 + 1] << 8)) *
+					h = (pb[m4] ^ ((ulong)pb[m4 + 1] << 8) ^ h) *
 						0x6C45CB2537A4271DUL;
 					break;
 				case 3:
 					Debug.Assert(m4 == (m - 3));
-					h = (h ^ pb[m4] ^ ((ulong)pb[m4 + 1] << 8) ^
-						((ulong)pb[m4 + 2] << 16)) * 0x59B8E8939E19695DUL;
+					h = (pb[m4] ^ ((ulong)pb[m4 + 1] << 8) ^
+						((ulong)pb[m4 + 2] << 16) ^ h) * 0x59B8E8939E19695DUL;
 					break;
 				default:
 					Debug.Assert(m4 == m);
@@ -612,14 +616,11 @@ namespace KeePassLib.Utility
 		{
 			if(s == null) throw new ArgumentNullException("s");
 
-			byte[] pb;
 			using(MemoryStream ms = new MemoryStream())
 			{
-				MemUtil.CopyStream(s, ms);
-				pb = ms.ToArray();
+				CopyStream(s, ms);
+				return ms.ToArray();
 			}
-
-			return pb;
 		}
 
 		public static byte[] Read(Stream s, int nCount)
@@ -648,6 +649,17 @@ namespace KeePassLib.Utility
 			return pb;
 		}
 
+		internal static string ReadString(Stream s, Encoding enc)
+		{
+			if(s == null) throw new ArgumentNullException("s");
+			if(enc == null) throw new ArgumentNullException("enc");
+
+			using(StreamReader sr = new StreamReader(s, enc, true))
+			{
+				return sr.ReadToEnd();
+			}
+		}
+
 		public static void Write(Stream s, byte[] pbData)
 		{
 			if(s == null) { Debug.Assert(false); return; }
@@ -662,7 +674,6 @@ namespace KeePassLib.Utility
 			if(pbData == null) throw new ArgumentNullException("pbData");
 			if(pbData.Length == 0) return pbData;
 
-			byte[] pbCompressed;
 			using(MemoryStream msSource = new MemoryStream(pbData, false))
 			{
 				using(MemoryStream msCompressed = new MemoryStream())
@@ -670,14 +681,12 @@ namespace KeePassLib.Utility
 					using(GZipStream gz = new GZipStream(msCompressed,
 						CompressionMode.Compress))
 					{
-						MemUtil.CopyStream(msSource, gz);
+						CopyStream(msSource, gz);
 					}
 
-					pbCompressed = msCompressed.ToArray();
+					return msCompressed.ToArray();
 				}
 			}
-
-			return pbCompressed;
 		}
 
 		public static byte[] Decompress(byte[] pbCompressed)
@@ -685,7 +694,6 @@ namespace KeePassLib.Utility
 			if(pbCompressed == null) throw new ArgumentNullException("pbCompressed");
 			if(pbCompressed.Length == 0) return pbCompressed;
 
-			byte[] pbData;
 			using(MemoryStream msData = new MemoryStream())
 			{
 				using(MemoryStream msCompressed = new MemoryStream(pbCompressed, false))
@@ -693,14 +701,12 @@ namespace KeePassLib.Utility
 					using(GZipStream gz = new GZipStream(msCompressed,
 						CompressionMode.Decompress))
 					{
-						MemUtil.CopyStream(gz, msData);
+						CopyStream(gz, msData);
 					}
 				}
 
-				pbData = msData.ToArray();
+				return msData.ToArray();
 			}
-
-			return pbData;
 		}
 
 		public static int IndexOf<T>(T[] vHaystack, T[] vNeedle)
@@ -710,10 +716,13 @@ namespace KeePassLib.Utility
 			if(vNeedle == null) throw new ArgumentNullException("vNeedle");
 			if(vNeedle.Length == 0) return 0;
 
-			for(int i = 0; i <= (vHaystack.Length - vNeedle.Length); ++i)
+			int cN = vNeedle.Length;
+			int iMax = vHaystack.Length - cN;
+
+			for(int i = 0; i <= iMax; ++i)
 			{
 				bool bFound = true;
-				for(int m = 0; m < vNeedle.Length; ++m)
+				for(int m = 0; m < cN; ++m)
 				{
 					if(!vHaystack[i + m].Equals(vNeedle[m]))
 					{
@@ -833,6 +842,19 @@ namespace KeePassLib.Utility
 			return true;
 		}
 
+		internal static int Count(byte[] pb, byte bt)
+		{
+			if(pb == null) { Debug.Assert(false); return 0; }
+
+			int cb = pb.Length, r = 0;
+			for(int i = 0; i < cb; ++i)
+			{
+				if(pb[i] == bt) ++r;
+			}
+
+			return r;
+		}
+
 		[MethodImpl(MioNoOptimize)]
 		internal static void DisposeIfPossible(object o)
 		{
@@ -842,6 +864,33 @@ namespace KeePassLib.Utility
 			if(d != null) d.Dispose();
 		}
 
+		internal static object GetEnumValue(Type tEnum, string strName)
+		{
+			if(tEnum == null) { Debug.Assert(false); return null; }
+			if(!tEnum.IsEnum) { Debug.Assert(false); return null; }
+			if(string.IsNullOrEmpty(strName)) { Debug.Assert(false); return null; }
+
+			return ((Array.IndexOf<string>(Enum.GetNames(tEnum), strName) >= 0) ?
+				Enum.Parse(tEnum, strName) : null);
+		}
+
+		internal static T ConvertObject<T>(object o, T tDefault)
+		{
+			if(o == null) return tDefault;
+
+			try
+			{
+				if(o is T) return (T)o;
+				return (T)Convert.ChangeType(o, typeof(T));
+			}
+			catch(Exception) { Debug.Assert(false); }
+
+			try { return (T)o; }
+			catch(Exception) { Debug.Assert(false); }
+
+			return tDefault;
+		}
+
 		internal static T BytesToStruct<T>(byte[] pb, int iOffset)
 			where T : struct
 		{
@@ -887,6 +936,25 @@ namespace KeePassLib.Utility
 
 			return pb;
 		}
+
+		internal static IntPtr AddPtr(IntPtr p, long cb)
+		{
+			// IntPtr.operator+ and IntPtr.Add are not available in .NET 2.0
+
+			if(IntPtr.Size >= 8)
+				return new IntPtr(unchecked(p.ToInt64() + cb));
+			return new IntPtr(unchecked(p.ToInt32() + (int)cb));
+		}
+
+		// Cf. Array.Empty<T>() of .NET 4.6
+		private static class EmptyArrayEx<T>
+		{
+			internal static readonly T[] Instance = new T[0];
+		}
+		internal static T[] EmptyArray<T>()
+		{
+			return EmptyArrayEx<T>.Instance;
+		}
 	}
 
 	internal sealed class ArrayHelperEx<T> : IEqualityComparer<T[]>, IComparer<T[]>
